class Action:
    def __init__(self, account, name, actor, permission, data):
        self.account = account
        self.name = name
        self.authorization = [{
            'actor': actor,
            'permission': permission
        }]
        self.data = data




# Wallet API removed from v1.2.0
WALLET_OPEN = '/v1/wallet/open'
WALLET_UNLOCK = 'v1/wallet/unlock'
WALLET_LOCK = 'v1/wallet/lock'
WALLET_SIGN_TRANSACTION = 'v1/wallet/sign_transaction'
WALLET_GET_PUBLIC_KEYS = 'v1/wallet/get_public_keys'

CHAIN_GET_INFO = 'v1/chain/get_info'
CHAIN_GET_BLOCK = 'v1/chain/get_block'
CHAIN_PUSH_TRANSACTION = 'v1/chain/push_transaction'
CHAIN_PUSH_TRANSACTIONS = 'v1/chain/push_transactions'
CHAIN_ABI_JSON_TO_BIN = 'v1/chain/abi_json_to_bin'
CHAIN_GET_REQUIRED_KEYS = 'v1/chain/get_required_keys'

HISTORY_GET_ACTIONS = 'v1/history/get_actions'
HISTORY_GET_TRANSACTION = 'v1/history/get_transaction'
HISTORY_GET_KEY_ACCOUNTS = 'v1/history/get_key_accounts'
HISTORY_GET_CONTROLLED_ACCOUNTS = 'v1/history/get_controlled_accounts'

NET_CONNECT = "v1/net/connect"
NET_DISCONNECT = "v1/net/disconnect"
NET_CONNECTIONS = "v1/net/connections"
NET_STATUS = "v1/net/status"

PRODUCER_PAUSE = "v1/producer/pause"
PRODUCER_RESUME = "v1/producer/resume"
PRODUCER_PAUSED = "v1/producer/paused"
PRODUCER_GET_RUNTIME_OPTIONS = "v1/producer/get_runtime_options"
PRODUCER_UPDATE_RUNTIME_OPTIONS = "v1/producer/update_runtime_options"
#PRODUCER_GET_GREYLIST = "v1/producer/get_greylist"
#PRODUCER_ADD_GREYLIST_ACCOUNTS = "v1/producer/add_greylist_accounts"
#PRODUCER_REMOVE_GREYLIST_ACCOUNTS = "v1/producer/remove_greylist_accounts"
#PRODUCER_GET_WHITELISTT_BLACKLIST = "v1/producer/remove_greylist_accounts"

DB_SIZE_GET = "v1/db_size/get"



import datetime

from . import utils


class TransactionBuilder:
    def __init__(self, wrapper):
        self.eos_client = wrapper

    @staticmethod
    def get_action(account, name, actor, permission, data):
        return {
            "account": account,
            "name": name,
            "authorization": [{
                "actor": actor,
                "permission": permission
            }
            ],
            "data": data
        }

    @staticmethod
    def get_transaction(
            expiration, ref_block_num, ref_block_prefix, max_net_usage_words=0, max_cpu_usage_ms=0, delay_sec=0,
            context_free_actions=None, actions=None, transaction_extensions=None, signatures=None,
            context_free_data=None):

        if not context_free_actions:
            context_free_actions = []
        if not actions:
            actions = []
        if not transaction_extensions:
            transaction_extensions = []
        if not signatures:
            signatures = []
        if not context_free_data:
            context_free_data = []

        return {
            "expiration": utils.datetime_to_eos_timestamp(expiration),
            "ref_block_num": ref_block_num,
            "ref_block_prefix": ref_block_prefix,
            "max_net_usage_words": max_net_usage_words,
            "max_cpu_usage_ms": max_cpu_usage_ms,
            "delay_sec": delay_sec,
            "context_free_actions": context_free_actions,
            "actions": actions,
            "transaction_extensions": transaction_extensions,
            "signatures": signatures,
            "context_free_data": context_free_data
        }

    def build_sign_transaction_request(self, actions):
        is_ok, get_info = self.eos_client.chain_get_info()
        ref_block_num = get_info['last_irreversible_block_num']
        is_ok, ref_block = self.eos_client.chain_get_block(ref_block_num)
        ref_block_prefix = ref_block['ref_block_prefix']
        timestamp = utils.eos_timestamp_to_datetime(ref_block['timestamp'])
        expiration = timestamp + datetime.timedelta(minutes=10)  # hardcoded ten minutes to expire
        chain_id = get_info['chain_id']

        actions = [e.__dict__ for e in actions]

        return TransactionBuilder.get_transaction(expiration, ref_block_num, ref_block_prefix, actions=actions), chain_id



from .json_rpc_wrapper import RPCWrapper
from .transaction_builder import TransactionBuilder
from .action import Action


class Suites(object):
    def __init__(self, wrapper, wallet_pass, public_key):
        self.rpc = wrapper
        self.wallet_pass = wallet_pass
        self.public_key = public_key

    def push_action(self, code, action_name, account, permission, params):
        try:
            self.rpc.wallet_unlock(self.wallet_pass)
        except:
            pass

        txBuilder = TransactionBuilder(self.rpc)
        is_ok, bin_param = self.rpc.chain_abi_json_to_bin(params)
        act = Action(code, action_name, account, permission, bin_param['binargs'])
        ready_tx, chain_id =  txBuilder.build_sign_transaction_request([act])
        is_ok, signed_transaction = self.rpc.wallet_sign_transaction(ready_tx, [self.public_key], chain_id)
        is_ok, ret = self.rpc.chain_push_transaction(signed_transaction)

        return ret



from .json_rpc_wrapper import RPCWrapper
from .suites import Suites

def init(api_endpoint, wallet_endpoint, wallet_key, public_key):
    global suite
    suite = Suites(
               RPCWrapper(api_endpoint, wallet_endpoint),
               wallet_key,
               public_key
            )



import json
import requests
import urllib.parse

from . import endpoints


class RPCWrapper(object):
    def __init__(self, api_endpoint, wallet_endpoint):
        self.api_endpoint = api_endpoint
        self.wallet_endpoint = wallet_endpoint

    def request(self, endpoint, uri, body=None):
        url = urllib.parse.urljoin(endpoint, uri)

        payloads = ""
        if body:
            payloads = json.dumps(body)
        else:
            payloads = None

        headers = {"Content-Type": "application/json"}
        if body:
            resp_obj = requests.post(url, data=payloads, headers=headers)
        else:
            resp_obj = requests.post(url, headers=headers)

        is_ok = True if resp_obj.status_code == 200 else False
        resp = resp_obj.json()
        return is_ok, resp

    def api_request(self, uri, body=None):
        return self.request(self.api_endpoint, uri, body)

    def wallet_request(self, uri, body=None):
        return self.request(self.wallet_endpoint, uri, body)

    # Wallet
    def wallet_lock(self, wallet='default'):
        return self.wallet_request(endpoints.WALLET_LOCK, wallet)

    def wallet_unlock(self, key, wallet='default'):
        return self.wallet_request(endpoints.WALLET_UNLOCK, [wallet, key])

    def wallet_open(self, wallet='default'):
        return self.wallet_request(endpoints.WALLET_OPEN, wallet)

    def wallet_get_public_keys(self):
        return self.wallet_request(endpoints.WALLET_GET_PUBLIC_KEYS)

    def wallet_sign_transaction(self, transaction, public_keys, chain_id):
        return self.wallet_request(
            endpoints.WALLET_SIGN_TRANSACTION, [transaction, public_keys, chain_id])

    # Chain
    def chain_get_info(self):
        return self.api_request(endpoints.CHAIN_GET_INFO)

    def chain_get_block(self, num_or_id):
        return self.api_request(endpoints.CHAIN_GET_BLOCK, {'block_num_or_id': num_or_id})

    def chain_abi_json_to_bin(self, abi_args):
        return self.api_request(endpoints.CHAIN_ABI_JSON_TO_BIN, abi_args)

    def chain_get_required_keys(self, transaction, available_keys):
        return self.api_request(endpoints.CHAIN_GET_REQUIRED_KEYS, {
            'transaction': transaction,
            'available_keys': available_keys
        })

    def chain_push_transaction(self, transaction):
        return self.api_request(endpoints.CHAIN_PUSH_TRANSACTION, {
            'transaction': transaction,
            'compression': 'none',
            'signatures': transaction['signatures']
        })

    # History
    def get_actions(self, position, offset, account_name):
        return self.api_request(endpoints.HISTORY_GET_ACTIONS, {
            'pos': position,
            'offset': offset,
            'account_name': account_name
        })

    def get_transaction(self, tx_id):
        return self.api_request(endpoints.HISTORY_GET_TRANSACTION, {
            'id': tx_id
        })

    def get_key_accounts(self, public_key):
        return self.api_request(endpoints.HISTORY_GET_KEY_ACCOUNTS, {
            'public_key': public_key
        })

    def get_controlled_accounts(self, controlling_account):
        return self.api_request(endpoints.HISTORY_GET_KEY_ACCOUNTS, {
            'controlling_account': controlling_account
        })

    # Net: Not yet implemented

    # Producer
    def pause(self):
        return self.api_request(endpoints.PRODUCER_PAUSE)

    def paused(self):
        return self.api_request(endpoints.PRODUCER_PAUSED)

    def resume(self):
        return self.api_request(endpoints.PRODUCER_RESUME)

    def get_runtime_options(self):
        return self.api_request(endpoints.PRODUCER_GET_RUNTIME_OPTIONS)

    def update_runtime_options(self):
        return self.api_request(endpoints.PRODUCER_UPDATE_RUNTIME_OPTIONS)

    # Db
    def db_size_get(self):
        return self.api_request(endpoints.DB_SIZE_GET)




from setuptools import setup

setup(name='pyeosio',
      version='0.1.0',
      description='EOSIO node connector for Python',
      url='https://github.com/psy2848048/pyeosio',
      author='Junhang (Bryan) RHEE',
      author_email='psy2848048@gmail.com',
      license='MIT',
      packages=['pyeosio'],
      install_requires=[
          'requests',
      ],
      zip_safe=False)



import datetime


def eos_timestamp_to_datetime(timestamp):
    if '.' in timestamp:
        timestamp, timedelta_ms = timestamp.split('.')
    else:
        timedelta_ms = 0

    timestamp = datetime.datetime.strptime(timestamp, '%Y-%m-%dT%H:%M:%S')
    timedelta_ms = datetime.timedelta(milliseconds=int(timedelta_ms))

    return timestamp + timedelta_ms


def datetime_to_eos_timestamp(dt):
    timestamp = dt.strftime('%Y-%m-%dT%H:%M:%S')
    if dt.microsecond:
        timedelta_ms = '{}'.format(int(dt.microsecond / 1000))
        timedelta_ms = timedelta_ms.zfill(3)
        return '{}.{}'.format(timestamp, timedelta_ms)
    return timestamp
